import { existsSync, readFileSync, lstatSync } from 'node:fs';
import path from 'node:path';
import { Command, Option, InvalidArgumentError } from 'commander';
import { Workspace, LogLevel, MAX_CLIENT_COUNT } from '../../types/index.js';
import { startLwcDevServer } from '../../server/index.js';
import ConsoleLogger from '../../server/consoleLogger.js';
import { templateString } from '../../utils/utils.js';
import { MISSING_CONFIG_FILE, MISSING_CONFIG_FILE_MRT, HTTPS_PORT_INVALID } from '../../labels.js';
const consoleLogger = new ConsoleLogger(LogLevel.info);
function getPortOption() {
    return new Option('-p, --port <number>', `[number] set port`)
        .env('PORT')
        .argParser(parseNumArg)
        .makeOptionMandatory();
}
function getWorkspaceOption() {
    return new Option('-w, --workspace <workspace>', '[string] configured workspace for development')
        .choices(['internal', 'SalesforceCLI', 'mrt'])
        .default('internal');
}
function parseNumArg(value) {
    const parsedValue = parseInt(value, 10);
    if (isNaN(parsedValue)) {
        throw new InvalidArgumentError('Not a number.');
    }
    return parsedValue;
}
function configExists(configFilePath) {
    return existsSync(configFilePath) && lstatSync(configFilePath).isFile();
}
function readConfig(configFile, workspace) {
    const parsedConfig = JSON.parse(readFileSync(configFile, 'utf-8'));
    let rootDir, lwc, https, identityToken;
    consoleLogger.log(LogLevel.debug, `Found config file at "${configFile}" and initializing dev server with it.`);
    if (workspace === Workspace.Mrt && path.basename(configFile).startsWith('lwc')) {
        lwc = parsedConfig;
    }
    else {
        rootDir = parsedConfig.rootDir;
        lwc = parsedConfig.lwc;
        https = parsedConfig.https;
        identityToken = parsedConfig.identityToken;
    }
    return {
        rootDir,
        lwc,
        https,
        identityToken,
    };
}
/**
 * SalesforceCLI workspaces don't require a config file to be present
 * @returns config absolute path | undefined
 */
function getDefaultConfig(rootDir, workspace) {
    if (workspace === Workspace.Internal) {
        const configFile = path.resolve(rootDir, 'lwc.config.json');
        if (configExists(configFile)) {
            return configFile;
        }
        const errorMsg = templateString(MISSING_CONFIG_FILE, [configFile]);
        consoleLogger.log(LogLevel.error, errorMsg);
        throw new Error(errorMsg);
    }
    else if (workspace === Workspace.Mrt) {
        // 1. we check for lwr.config
        const configFile = path.resolve(rootDir, 'lwr.config.json');
        if (configExists(configFile)) {
            return configFile;
        }
        // 2. fallback to lwc.config
        const configFile2 = path.resolve(rootDir, 'lwc.config.json');
        if (configExists(configFile2)) {
            return configFile2;
        }
        // 3. none of the configs are present
        const errorMsg = templateString(MISSING_CONFIG_FILE_MRT, [configFile, configFile2]);
        consoleLogger.log(LogLevel.error, errorMsg);
        throw new Error(errorMsg);
    }
}
function resolveModuleRootDir(rootDir, entryDir) {
    if (entryDir.startsWith('$rootDir')) {
        return entryDir.replace(/^\$rootDir/, rootDir);
    }
    else if (path.isAbsolute(entryDir)) {
        return entryDir;
    }
    else {
        // entryDir is a relative path
        return path.resolve(rootDir, entryDir);
    }
}
function normalizeConfig({ config, port, logLevel, workspace, maxClientCount = MAX_CLIENT_COUNT, }) {
    let rootDir = process.cwd();
    const configFile = config
        ? path.resolve(rootDir, config)
        : getDefaultConfig(rootDir, workspace);
    const modulePaths = [];
    const parsedConfig = configFile ? readConfig(configFile, workspace) : {};
    consoleLogger.log(LogLevel.debug, `Found config file at "${configFile}" and initializing dev server with it.`);
    rootDir = parsedConfig.rootDir ?? rootDir;
    if (parsedConfig.lwc?.modules) {
        parsedConfig.lwc.modules.forEach((entry) => {
            if (entry.dir) {
                const resolvedDir = resolveModuleRootDir(rootDir, entry.dir);
                if (existsSync(resolvedDir)) {
                    modulePaths.push(resolvedDir);
                    consoleLogger.log(LogLevel.debug, `FS-Watcher: Adding ${resolvedDir} to watch.`);
                }
                else {
                    consoleLogger.log(LogLevel.warn, `FS-Watcher: ${resolvedDir} does not exist, ignoring.`);
                }
            }
        });
    }
    const normalizedConfig = {
        rootDir,
        port,
        // Component paths to watch
        paths: modulePaths,
        workspace,
        logLevel,
        maxClientCount,
        identityToken: parsedConfig.identityToken,
    };
    if (parsedConfig.https) {
        // @ts-expect-error : typescript is unable to deduce that if we are here then configFile exists
        normalizedConfig.https = parseHttpsConfig(parsedConfig.https, path.dirname(configFile));
    }
    return normalizedConfig;
}
function parseHttpsConfig(config, configDir) {
    if (typeof config !== 'object')
        throw new Error('Expected config to be an object');
    if (config === null)
        throw new Error(`config can't be null`);
    let cert, key, pfx, passphrase, port;
    // could solve this repetition, but I don't know how to resolve typescript
    if ('cert' in config && typeof config.cert === 'string') {
        cert = readFileSync(path.resolve(configDir, config.cert), 'utf-8');
    }
    if ('key' in config && typeof config.key === 'string') {
        key = readFileSync(path.resolve(configDir, config.key), 'utf-8');
    }
    if ('pfx' in config && typeof config.pfx === 'string') {
        pfx = readFileSync(path.resolve(configDir, config.pfx));
    }
    if ('passphrase' in config && typeof config.passphrase === 'string') {
        passphrase = config.passphrase;
    }
    if ('port' in config && typeof config.port === 'number') {
        port = config.port;
    }
    else {
        consoleLogger.log(LogLevel.error, HTTPS_PORT_INVALID);
        throw new Error(HTTPS_PORT_INVALID);
    }
    return { cert, key, pfx, passphrase, port };
}
function addExitHandler(cb) {
    ['exit', 'SIGTERM', 'SIGINT'].forEach((signal) => {
        process.on(signal, () => {
            cb();
        });
    });
}
export function createDevServer() {
    return new Command('start')
        .aliases(['preview'])
        .description('Start Dev Server')
        .addOption(getPortOption())
        .addOption(getWorkspaceOption())
        .action(async (_options, cmd) => {
        const { config, port, logLevel, workspace } = cmd.optsWithGlobals();
        const serverConfig = normalizeConfig({
            config,
            port,
            logLevel: LogLevel[logLevel],
            workspace,
        });
        const lwcDevServer = await startLwcDevServer(serverConfig);
        addExitHandler(() => {
            lwcDevServer.stopServer();
        });
    });
}
//# sourceMappingURL=devServer.js.map