/*
 * Copyright (c) 2018, salesforce.com, inc.
 * All rights reserved.
 * SPDX-License-Identifier: MIT
 * For full license text, see the LICENSE file in the repo root or https://opensource.org/licenses/MIT
 */
import * as http from 'http';
import * as https from 'https';
import { URL } from 'node:url';
import { WebSocketServer } from 'ws';
import { LogLevel } from '../../types/index.js';
import { templateString } from '../../utils/utils.js';
import { WSS_STARTUP_FAILURE } from '../../labels.js';
/**
 * Special mechanism to take internal users through an authentication flow.
 */
function requestHandler(req, res) {
    // the second arg could really be anything, we're just interested in searchParams
    // also req.url can never be undefined, it's just bad typing
    const url = new URL(req.url, 'http://localhost');
    if (req.method !== 'GET' || url.pathname !== '/') {
        res.writeHead(404, { 'Content-Type': 'text/html' });
        return res.end(`404 Not Found`);
    }
    const redirectUrl = url.searchParams.get('redirectUrl');
    if (redirectUrl) {
        res.statusCode = 302;
        res.setHeader('Location', redirectUrl);
        return res.end();
    }
    res.writeHead(200, { 'Content-Type': 'text/html' });
    res.end(`Visit https://&lt;salesforce-url&gt;?0.aura.mode=devpreview`);
}
const TRUSTED_ORIGINS = [
    '\\.sfdcdev\\.force\\.com:[0-9]+$',
    '\\.w[a-z]\\.crm\\.dev:6101$',
    '\\.lightning\\.force\\.com$',
    '\\.lightning\\.pc-rnd\\.force\\.com$',
];
function isFromTrustedOrigin(request) {
    const { headers } = request;
    const { origin } = headers;
    return (
    // Only upgrade requests that are coming in from the hmr-client
    // https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications#protocols
    headers['sec-websocket-protocol'] === 'lightning-dev-preview' &&
        origin &&
        TRUSTED_ORIGINS.some((trustedOrigin) => new RegExp(trustedOrigin).test(origin)));
}
/**
 * Start a WebSocketServer.
 *
 * Note: Based on examples in the ws git repo: https://github.com/websockets/ws/blob/master/examples/express-session-parse/index.js
 * @param param0
 * @param logger
 * @returns
 */
export function startWebSocketServer({ port, https: httpsConfig }, logger) {
    try {
        // Start a WebSocketServer disconnected from the http server, allows us to handle authantication before upgrading ws connections
        const wss = new WebSocketServer({ noServer: true });
        const httpServer = http.createServer();
        httpServer.maxRequestsPerSocket = 1;
        // Special handler for the internal usecase
        httpServer.on('request', requestHandler);
        const upgradeHandler = (request, socket, head) => {
            if (!isFromTrustedOrigin(request)) {
                socket.write('HTTP/1.1 401 Unauthorized\r\n\r\n');
                socket.destroy();
                return;
            }
            wss.handleUpgrade(request, socket, head, function (ws) {
                wss.emit('connection', ws, request);
            });
        };
        // Handle upgrade of connections after verifying the validity
        httpServer.on('upgrade', upgradeHandler);
        httpServer.listen(port, () => {
            logger.log(LogLevel.info, `[LWC Dev Server] LWC Dev Server started at port:${port}`);
        });
        if (httpsConfig) {
            const httpsServer = https.createServer(httpsConfig);
            httpsServer.on('request', requestHandler);
            httpsServer.on('upgrade', upgradeHandler);
            httpsServer.listen(httpsConfig.port, () => {
                logger.log(LogLevel.info, `[LWC Dev Server] LWC HTTPS Dev Server started at port:${httpsConfig.port}`);
            });
            wss.on('close', () => {
                httpsServer.close();
            });
        }
        // When the WebSocketServer is being closed, close the http server as well.
        wss.on('close', () => {
            httpServer.close();
        });
        return wss;
    }
    catch (e) {
        const errorMsg = templateString(WSS_STARTUP_FAILURE, [e]);
        logger.log(LogLevel.error, errorMsg);
        throw e;
    }
}
//# sourceMappingURL=web-socket-server.js.map