import { describe, it, expect, vi, beforeEach, afterEach } from 'vitest';
import { FSWatcher } from 'chokidar';
import { LogLevel } from '../types/index.js';
import { LWCServer } from './index.js';
import { startWatch } from './watcher/index.js';
import { AuthenticatedWebSocketServer } from './connection/web-socket-server.js';
import { ModuleGraph } from './moduleGraph/index.js';
import ConsoleLogger from './consoleLogger.js';
import { Client } from './connection/client.js';
import { Connection } from './connection/connection.js';
vi.mock('chokidar', () => ({ FSWatcher: vi.fn(() => ({ on: vi.fn(), close: vi.fn() })) }));
vi.mock('./watcher/index.js', () => ({ startWatch: vi.fn(() => new FSWatcher()) }));
vi.mock('./connection/web-socket-server.js', () => ({
    AuthenticatedWebSocketServer: vi.fn(() => ({
        onNewConnection: vi.fn(),
        close: vi.fn(),
    })),
}));
vi.mock('./moduleGraph/index.js', () => ({ ModuleGraph: vi.fn() }));
vi.mock('./consoleLogger.js');
vi.mock('./connection/client.js', () => ({
    Client: vi.fn(() => ({ onClose: vi.fn(), send: vi.fn() })),
}));
vi.mock('./connection/connection.js');
describe('LWCServer', () => {
    let config;
    let logger;
    let wss;
    let lwcServer;
    beforeEach(() => {
        config = {
            paths: ['/some/path'],
            logLevel: LogLevel.info,
            identityToken: 'test-token',
            maxClientCount: 2,
        };
        logger = new ConsoleLogger(config.logLevel);
        wss = new AuthenticatedWebSocketServer(config, logger);
        lwcServer = new LWCServer(config, wss, logger);
    });
    afterEach(() => {
        vi.clearAllMocks();
    });
    it('should initialize the server with correct components', () => {
        expect(startWatch).toHaveBeenCalledWith(config.paths);
        expect(ModuleGraph).toHaveBeenCalledWith(config, logger, lwcServer.watcher, {
            moduleUpdateHandler: lwcServer.moduleUpdateHandler,
        });
    });
    it('should handle new connections and enforce max client limit', () => {
        const mockUrl = 'ws://localhost';
        const onNewConnection = vi.mocked(wss.onNewConnection).mock.calls[0][0];
        // @ts-expect-error
        const connection = new Connection();
        onNewConnection(connection, mockUrl);
        expect(Client).toHaveBeenCalledTimes(1);
        expect(lwcServer.clients.size).toBe(1);
        onNewConnection(connection, mockUrl);
        onNewConnection(connection, mockUrl); // This should exceed max client count
        expect(Client).toHaveBeenCalledTimes(2); // Only two clients allowed
        expect(connection.close).toHaveBeenCalledTimes(1); // Third connection should be closed
        expect(lwcServer.clients.size).toBe(2);
    });
    it('should handle file watcher errors and stop the server', () => {
        const mockErrorEvent = { message: 'File watcher error' };
        const onError = vi
            .mocked(lwcServer.watcher)
            .on.mock.calls.find(([eventName]) => eventName === 'error')[1];
        onError(mockErrorEvent);
        expect(logger.log).toHaveBeenCalledWith(LogLevel.error, '[LWC Dev Server] Encountered error while watching files');
        expect(logger.log).toHaveBeenCalledWith(LogLevel.debug, `[LWC Dev Server] File watcher error: ${mockErrorEvent.message}`);
        expect(wss.close).toHaveBeenCalled();
    });
    it('should stop the server and release resources', () => {
        lwcServer.stopServer();
        expect(logger.log).toHaveBeenCalledWith(LogLevel.info, '[LWC Dev Server] Stopping the dev server');
        expect(lwcServer.watcher.close).toHaveBeenCalled();
        expect(lwcServer.wss.close).toHaveBeenCalled();
        expect(logger.log).toHaveBeenCalledWith(LogLevel.info, '[LWC Dev Server] Succesfully stopped dev server');
    });
    it('should send update messages to all connected clients', () => {
        const mockClient1 = new Client('client1', 'ws://localhost', {}, lwcServer.context);
        const mockClient2 = new Client('client2', 'ws://localhost', {}, lwcServer.context);
        lwcServer.clients.add(mockClient1);
        lwcServer.clients.add(mockClient2);
        const modulePath = '/some/module/path';
        lwcServer.moduleUpdateHandler(modulePath);
        expect(mockClient1.send).toHaveBeenCalledWith({
            type: 'update',
            data: [{ modulePath }],
        });
        expect(mockClient2.send).toHaveBeenCalledWith({
            type: 'update',
            data: [{ modulePath }],
        });
    });
});
//# sourceMappingURL=index.spec.js.map