import fs from 'fs';
import path from 'path';
import { LogLevel } from '../../types/index.js';
import { getModuleHash, readModule } from '../util/fileUtil.js';
import { getWorkspaceScanner } from './workspaceScanner/index.js';
export class ModuleGraph {
    constructor(config, logger, watcher, moduleChangeCallbacks) {
        /**
         * All LWC modules and their parts with absolute file paths, for the current workspace.
         */
        this.moduleMap = new Map();
        /**
         * All modules paths that are being used by the various client connections
         */
        this.activeModulePaths = [];
        // Mapping from <absolute file path> => "<namespace>/<componentname>/component.js"
        this.fileToModulePathMap = Object.create(null);
        // Mapping from "<namespace>/<componentname>/component.js" => { moduleFilePath, moduleSourceHash }
        this.modulePathToModuleInfo = Object.create(null);
        this.watcher = watcher;
        this.logger = logger;
        this.moduleChangeCallbacks = moduleChangeCallbacks;
        this.workspaceScanner = getWorkspaceScanner(config, logger);
        // init() may be called multiple times in the future(when a directory is deleted or addeds)
        this.init();
    }
    init() {
        this.moduleMap = this.workspaceScanner.getModulesMap();
        if (this.moduleMap.size) {
            this.logger.log(LogLevel.debug, `[Module Graph] Found ${this.moduleMap.size} LWC components in the current workspace.`);
        }
        else {
            this.logger.log(LogLevel.debug, `[Module Graph] Found no LWC components in the current workspace, check your settings in lwc.config.json.`);
        }
        this.moduleMap.forEach(({ moduleParts }) => {
            Object.assign(this.modulePathToModuleInfo, moduleParts);
            for (const [modulePath, moduleInfo] of Object.entries(moduleParts)) {
                this.fileToModulePathMap[moduleInfo.moduleFilePath] = modulePath;
            }
        });
        this.watcher.on('change', this.fileChangeHandler.bind(this));
    }
    // Given absolute file path, get the modulePath
    // Mapping from <absolute file path> => "<namespace>/<componentname>/component.js"
    getModulePathByFile(path) {
        return this.fileToModulePathMap[path];
    }
    // Given the modulePath, get the absolute file path for that module
    getFileByModulePath(modulePath) {
        return this.modulePathToModuleInfo[modulePath]?.moduleFilePath;
    }
    /**
     * Register the active paths that the client is interested in
     * @param moduleRegistration
     * @returns Modules paths that are determined to be stale wrt the content monitored by the dev server
     */
    registerActivePaths(moduleRegistration) {
        const { data: { modules: activeModules }, } = moduleRegistration;
        const staleModules = [];
        const unTrackedModules = [];
        activeModules.forEach((mod) => {
            const { modulePath, hash } = mod;
            if (modulePath.trim()) {
                const moduleInfo = this.modulePathToModuleInfo[modulePath];
                if (!moduleInfo) {
                    unTrackedModules.push(modulePath);
                }
                else {
                    this.activeModulePaths.push(modulePath);
                    if (moduleInfo.moduleSourceHash !== hash) {
                        staleModules.push(modulePath);
                    }
                }
            }
        });
        unTrackedModules.length &&
            this.logger.log(LogLevel.verbose, `[Module Graph] Following modules are unknown to dev server: \n\t ${unTrackedModules.join(',\n\t')}`);
        staleModules.length &&
            this.logger.log(LogLevel.debug, `[Module Graph] Following modules have a newer version: \n\t ${staleModules.join(',\n\t')}`);
        // TODO: Before returning staleModules, the list can be trimmed based on module boundary
        // For example ['c/mycomponent/mycomponent.js', 'c/mycomponent/mycomponent.html', 'c/mycomponent/mycomponent.css] -> ['c/mycomponent/mycomponent.js']
        // because 'c/mycomponent/mycomponent.js' module update will contain full bundle
        return staleModules;
    }
    fileChangeHandler(filePath) {
        const changedFile = path.resolve(filePath);
        if (fs.lstatSync(changedFile).isFile()) {
            const modulePath = this.getModulePathByFile(changedFile);
            if (modulePath) {
                const moduleInfo = this.modulePathToModuleInfo[modulePath];
                const newModuleHash = getModuleHash(changedFile);
                if (moduleInfo.moduleSourceHash === newModuleHash) {
                    return;
                }
                this.logger.log(LogLevel.info, `[LWC Dev Server] File '${modulePath}' modified - updating.`);
                moduleInfo.moduleSourceHash = newModuleHash;
                // Find the nearest boundary for the file that changed
                const moduleBoundary = this.getModuleBoundary(modulePath);
                // Notify the clients that a module has been updated
                this.moduleChangeCallbacks.moduleUpdateHandler(moduleBoundary);
            }
        }
    }
    /**
     * Identify which modulePath that represents the boundary of the changed module.
     * In the case of .html files, the module boundary is the template file itself. swapTemplate()
     *  is called and module swap is done.
     * In the case of .css files, the module boundary is the css file itself, swapStyle() is called
     *  and module swap is done.
     * In the case of .js files, the module boundary is the main entry file of the module, all the files
     *  reachable from the entry module are bundled and swapComponent invoked.
     *  Since JS hot swap is currently supporting components, this focused update of the main entry file
     *  is acceptable.
     * @param modulePath The module that changed
     * @return The modulePath of the boundary that needs to be updated.
     */
    getModuleBoundary(modulePath) {
        const { ext } = path.parse(modulePath);
        if (['.html', '.css'].includes(ext)) {
            return modulePath;
        }
        else if (['.js', '.ts'].includes(ext)) {
            const ownerModuleId = this.modulePathToModuleInfo[modulePath].ownerModuleId;
            const moduleMapEntry = this.moduleMap.get(ownerModuleId);
            if (moduleMapEntry) {
                const [namespace, name] = ownerModuleId.split('/');
                const { moduleParts } = moduleMapEntry;
                const entryFile = [
                    `${namespace}/${name}/${name}.js`,
                    `${namespace}/${name}/${name}.ts`,
                    `${namespace}/${name}/${name}.css`,
                ].find((entry) => Object.hasOwnProperty.call(moduleParts, entry));
                if (entryFile) {
                    return entryFile;
                }
                throw new Error(`[LWC Dev Server] Unable to determine entry file for path: ${modulePath}`);
            }
        }
        throw new Error(`[LWC Dev Server] Unrecognized file type or file not part of LWC module: ${modulePath}`);
    }
    /**
     * Fetch files and source related to the modulePath that is being hot loaded.
     * @param {string} name Name of the component being compiled
     * @param {string} namespace The namespace of the component
     * @param {string} modulePath The formatted path for the file being compiled.
     * @returns
     */
    getModuleFiles(name, namespace, modulePath) {
        const { ext } = path.parse(modulePath);
        const moduleDir = `${namespace}/${name}`;
        if (['.html', '.css'].includes(ext)) {
            const moduleFilePath = this.getFileByModulePath(modulePath);
            const fileEntry = path.relative(moduleDir, modulePath);
            // It can be assumed that the module file exists and the source can be read
            const src = readModule(moduleFilePath);
            return { [fileEntry]: src };
        }
        else if (['.js', '.ts'].includes(ext)) {
            const ownerModuleId = this.modulePathToModuleInfo[modulePath].ownerModuleId;
            const moduleMapEntry = this.moduleMap.get(ownerModuleId);
            return Object.keys(moduleMapEntry.moduleParts).reduce((acc, key) => {
                const part = path.relative(moduleDir, key);
                const partSrc = readModule(moduleMapEntry.moduleParts[key].moduleFilePath);
                acc[part] = partSrc;
                return acc;
            }, {});
        }
        throw new Error(`[LWC Dev Server] Unable to determine source for '${modulePath}'`);
    }
}
//# sourceMappingURL=index.js.map