import path from 'path';
import { describe, it, expect, vi, beforeEach, afterEach } from 'vitest';
import { watch } from 'chokidar';
import { LogLevel, Workspace } from '../../types/index.js';
import { getModuleHash } from '../util/fileUtil.js';
import { ModuleGraph } from './index.js';
const modulesPath = path.resolve(__dirname, 'workspaceScanner/__tests__/data/mrt/some-path');
vi.spyOn(path, 'resolve');
vi.mock('path', async (importOriginal) => {
    return {
        ...(await importOriginal()),
        resolve: vi.fn(),
    };
});
vi.mock('chokidar', () => ({
    watch: vi.fn().mockReturnValue({
        on: vi.fn(),
        close: vi.fn(),
    }),
}));
vi.mock('../util/fileUtil.js', async (importOriginal) => {
    return {
        ...(await importOriginal()),
        getModuleHash: vi.fn(),
    };
});
describe('ModuleGraph', () => {
    let moduleGraph;
    const mockLogger = {
        log: vi.fn(),
    };
    //@ts-expect-error mocked
    const mockWatcher = watch();
    const mockCallbacks = {
        moduleUpdateHandler: vi.fn(),
    };
    beforeEach(() => {
        moduleGraph = new ModuleGraph({
            logLevel: LogLevel.debug,
            paths: [modulesPath],
            port: 50182,
            rootDir: __dirname,
            workspace: Workspace.Mrt,
        }, mockLogger, mockWatcher, mockCallbacks);
    });
    afterEach(() => {
        vi.clearAllMocks();
    });
    it('should return the correct module path by file path', () => {
        const result = moduleGraph.getModulePathByFile(path.resolve(modulesPath, 'ns/comp/comp.html'));
        expect(result).toBe('ns/comp/comp.html');
    });
    it('should handle file changes correctly', () => {
        const mockFilePath = path.resolve(modulesPath, 'ns/comp/comp.js');
        vi.mocked(getModuleHash).mockReturnValueOnce('changed hash');
        vi.mocked(mockWatcher).on.mock.calls[0][1](mockFilePath);
        expect(mockCallbacks.moduleUpdateHandler).toHaveBeenCalled();
    });
    it('should throw an error for unrecognized file types in getModuleBoundary', () => {
        expect(() => {
            moduleGraph.getModuleBoundary('/path/to/file.txt');
        }).toThrowError('[LWC Dev Server] Unrecognized file type or file not part of LWC module: /path/to/file.txt');
    });
    it('should return the correct module boundary for .js files', () => {
        const modulePath = 'ns/comp/x.js';
        const result = moduleGraph.getModuleBoundary(modulePath);
        expect(result).toBe('ns/comp/comp.js');
    });
});
//# sourceMappingURL=index.spec.js.map